﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.ComponentModel;

namespace howto_clone_with_serialization
{
    [Serializable()]
    class Person : ICloneable
    {
        private string _FirstName;
        [Description("The person's first or given name")]
        [Category("Name")]
        public string FirstName
        {
            get
            {
                return _FirstName;
            }
            set
            {
                _FirstName = value;
            }
        }

        private string _LastName;
        [Description("The person's last or family name")]
        [Category("Name")]
        public string LastName
        {
            get
            {
                return _LastName;
            }
            set
            {
                _LastName = value;
            }
        }

        private StreetAddress _Address;
        [Description("The person's street address")]
        [Category("Contact Info")]
        public StreetAddress Address
        {
            get
            {
                return _Address;
            }
            set
            {
                _Address = value;
            }
        }

        private string _Email;
        [Description("The person's primary email address")]
        [Category("Contact Info")]
        public string Email
        {
            get
            {
                return _Email;
            }
            set
            {
                _Email = value;
            }
        }

        private string _Phone;
        [Description("The person's business phone number")]
        [Category("Contact Info")]
        public string Phone
        {
            get
            {
                return _Phone;
            }
            set
            {
                _Phone = value;
            }
        }

        // Return the Person as a string for display purposes.
        public override string ToString()
        {
            return FirstName + " " + LastName;
        }

        // Types of clones.
        public enum CloneType
        {
            Shallow,
            Deep
        }

        // Make a clone of the person.
        public Person Clone(CloneType clone_type)
        {
            // Make a shallow clone.
            Person new_person = (Person)MemberwiseClone();

            // If this should be a deep clone, make a new Address object.
            if (clone_type == CloneType.Deep)
            {
                new_person.Address = (StreetAddress)Address.Clone();
            }

            // Return the new Person.
            return new_person;
        }

        #region ICloneable Members

        // Make a deep clone of the person.
        public object Clone()
        {
            return Clone(CloneType.Deep);
        }

        #endregion
    }
}
